# frozen_string_literal: true

require_relative "masterdata/version"
require_relative "masterdata/dialcode"
require 'i18n_data'
require 'csv'
require "normalize_country"
require 'city-state'
require 'iso_country_codes'
require 'validates_zipcode'
require 'phonelib'
require 'email_address'
require 'geocoder'

module Masterdata
  class Error < StandardError; end
  # Your code goes here...
  class << self 
    
    def read_csv(path)
      data = {}
      df = CSV.table(path)
      df.headers.each do |column|
          data.merge!(column=>df[column].map{|values| values.class==NilClass ? "N" : values}.map(&:to_s))
      end
      return data
    end
    @@country_codes = I18nData.countries.keys().map(&:downcase)
    @@country_names = I18nData.countries.values().map(&:downcase)
    @@dialcode = Masterdata::dialcode_country
    def validate_abbrevate_countrycodes(csv_data,countrycodecol)
      alpha_country_codes = []
      abbre_country_codes = []
      csv_data[countrycodecol.to_sym].map(&:downcase).each do |alpha2|
           if @@country_codes.include?(alpha2[0,2])
                alpha_country_codes << alpha2[0,2]                # validating country codes
                abbre_country_codes << NormalizeCountry(alpha2[0,2].upcase,  :to => :short) #abbrevating CountryCodes after validation
           elsif alpha2=="N"
                alpha_country_codes << alpha2
                abbre_country_codes << alpha2
           else 
                alpha_country_codes << "I"   # invalid
                abbre_country_codes << "N"   # null value
           end
      end
      return alpha_country_codes , abbre_country_codes
    end

    def val_countrystate(csv_data,col,countrycol)
      col = col.map{|e| e.to_sym}
      state_names = []
      csv_data[countrycol.to_sym].map{|code| if code.size==2 then state_names << CS.states(code.downcase.to_sym).values() end} #states names getting
      country_state_names= (@@country_names << state_names.compact().flatten.map(&:downcase)).flatten  #collecting all country and state names
      # val_country_state_names = [:state,:country]
      col.each do |ele|
           csv_data[ele].each_with_index do |data,index|
                if data !="N"
                     if country_state_names.include?(data.downcase)
                          csv_data[ele][index] = data             #validating states and country names
                     else 
                          csv_data[ele][index] = "I"
                     end
                elsif data == "N"
                     csv_data[ele][index] = "N"
                else 
                     csv_data[ele][index]
                end
           end
      end
      return col.map{|e| csv_data[e]}
    end

    def val_dialcode(csv_data,dialcol)
      country_dail_codes = @@country_codes.map(&:upcase).map{|codes| IsoCountryCodes.find(codes).calling[1..-1]} # collecting all country dial codes from countries
      val_country_dail_codes = []
      csv_data[dialcol.to_sym].each do |i|
                if country_dail_codes.include?(i) 
                     val_country_dail_codes << "+#{i}"    # validating column dial codes
                elsif i == "N"
                     val_country_dail_codes << "N"
                else 
                     val_country_dail_codes << "I"
                end 
      end
      return val_country_dail_codes
    end

    def infogain_countrycode(csv_data,column)
          infogaincc = []
          csv_data[column.to_sym].each do |codes|
          # p codes
               if (codes =="I" || codes == "N")       #countrycodes
                    infogaincc << ["NO INFO",codes,"NO INFO"]
               else 
                    infogaincc << [NormalizeCountry(IsoCountryCodes.find(codes).name , :to => :short) , IsoCountryCodes.find(codes).calling]
               end 
          end
          return infogaincc
     end

     def infogain_country(csv_data,column)
          infogainc = []
          csv_data[column.to_sym].each do |codes|
          # p codes
               if (codes =="I" || codes == "N")          #country
                    infogainc << ["NO INFO","NO INFO"]
               else 
                    infogainc << [I18nData.country_code(codes),IsoCountryCodes.find(I18nData.country_code(codes)).calling]
               end 
          end
          return infogainc
     end

     def infogain_countrydialcode(csv_data,column)
          infogaincdc = []
          csv_data[column.to_sym].each do |codes|
          # p codes
               if (codes =="I" || codes == "N")          #countrydialcodes
                    infogaincdc << ["NO INFO","NO INFO"]
               else 
                    infogaincdc << [@@dialcode[codes],I18nData.country_code(@@dialcode[codes])]
               end 
          end
          return infogaincdc
     end

     def update_columns(csv_data,headers,header_values)
          headers = headers.map{|x| x.to_sym}
          headers.each_with_index do |data,index|
               csv_data[headers[index]].replace(header_values[index])
          end 
          return csv_data
     end

     def val_pincode(pincode_col,countrycode_col)
          pincodes = []
          pincode_col.each_with_index do |data,index|
              if data != "N" 
                    if ValidatesZipcode.valid?(data,countrycode_col[index])
                         pincodes << data 
                    else 
                         pincodes << "I"
                    end
              else 
                    pincodes << "N"
              end
          end
          return pincodes
     end 

     def val_contactnumber(contact_col,countrycode_col)
          valcontactnumber = []
          contact_col.each_with_index do |data,index|
               if Phonelib.valid_for_country?(data,countrycode_col[index])
                    valcontactnumber << data 
               else 
                    valcontactnumber << "I"
               end
          end
          return valcontactnumber
     end

     def val_emailaddress(emailcol)
          valemailcol = []
          emailcol.each do |val|
               # p val
               if val != "N"
                    if EmailAddress.valid?(val)
                         valemailcol << val 
                    else 
                         valemailcol << "I"
                    end 
               else 
                    valemailcol << "N"
               end
          end  
          return valemailcol
     end
  end
end
